function [Q, x, w]=GaussianLegendreQ(fhandle, a, b, N)

% This function is for computing definite integrals of fhandle using Gauss-Legendre 
% quadrature. Computes the Gauss-Legendre nodes and weights  on an interval
% [a,b] with N points rule and then get the quadrature.
N0=N-1;
N1=N; 
N2=N+1;

x0=linspace(-1,1,N1)';

% Initial guess
y=cos((2*(0:N0)'+1)*pi/(2*N0+2))+(0.5/N1)*sin(pi*x0*N0/N2);

% Gauss-Legendre Vandermonde Matrix
L=zeros(N1,N2);

% Derivative of LGVM
LP=zeros(N1,N2);

% Compute the roots of the N Legendre Polynomial
% using the recursion relation and the Newton-Raphson method
y0=2;

% Iterate until new points are uniformly within epsilon of old points
while max(abs(y-y0))>eps    
    L(:,1)=1;
    LP(:,1)=0;
    
    L(:,2)=y;
    LP(:,2)=1;
    
    for k=2:N1
        L(:,k+1)=( (2*k-1)*y.*L(:,k)-(k-1)*L(:,k-1) )/k;
    end
 
    LP=(N2)*( L(:,N1)-y.*L(:,N2) )./(1-y.^2);   
    
    y0=y;
    y=y0-L(:,N2)./LP;    
end

% Linear map from[-1,1] to [a,b]
x=(a*(1-y)+b*(1+y))/2;      

% Compute the weights
w=(b-a)./((1-y.^2).*LP.^2)*(N2/N1)^2;

% Compute the quadrature
Q=fhandle(x)'*w;