function [Rmplus1,theta]=analytical_Ez(M,n_thetas)
% analytical solution for Ez
% note: M here is the number of layers! but this doesn't really matter very
% much as the solution doesn't vary much with change in M for sufficiently
% large M.
c=299792458;
u0=4*pi*1e-7;
e0=1/(c^2*u0);
f=30e9;
lambda=c/f;
L=5*lambda;
x = linspace(0,L,M+1); %x(1) = 0, x(m+1) = L. the length, minus 1, is the number of layers. 
y = linspace((x(2)+x(1))/2, (x(M+1)+x(M))/2,M); %this is the center of each layer.
er = 4+(2-j*0.1)*(1-y/L).^2; %should be calculated in each layer, NOT at each node.
er(M+1) = 1;
ur = linspace(2-j*0.1,2-j*0.1,M);
ur(M+1) = 1;
angle = linspace(0,90,n_thetas)*pi/180;
k0=f*2*pi*sqrt(u0*e0);
for p = 1:n_thetas
    theta = angle(p);
    kx = f*2*pi*sqrt(u0*e0)*sqrt(ur.*er-sin(theta)^2);
    R(1) = -1;
    %epol
    for m =1:M-1 %to find the reflection coefficient at each boundary layer, before the boundary at the vacuum interface
        n(m+1) = (ur(m)*kx(m+1)-ur(m+1)*kx(m))./(ur(m)*kx(m+1)+ur(m+1)*kx(m));
        R(m+1) = (n(m+1)+R(m).*exp(-2*j*kx(m).*x(m+1)))/(1+n(m+1)*R(m).*exp(-2*j*kx(m).*x(m+1))).*exp(2*j*kx(m+1)*x(m+1));
    end
    for m = M %to find the reflection coefficient at the interface between the last dielectric layer and freespace
        n(m+1) = (ur(m)*kx(m+1)-ur(m+1)*kx(m))./(ur(m)*kx(m+1)+ur(m+1)*kx(m));
        R(m+1) = (n(m+1)+R(m).*exp(-2*j*kx(m).*x(m+1)))/(1+n(m+1)*R(m).*exp(-2*j*kx(m).*x(m+1))).*exp(2*j*kx(m+1)*x(m+1));
    end
    %now look at the dielectric/vacuum interface. 
    Rmplus1(p) = R(M+1);
%     % compute E-fields
%     A=zeros(1:M+1,1);
%     A(M+1)=1;
%     Ez(M+1,p)=exp(j*k0*x(M+1)*cos(theta))+R(M+1)*exp(-j*k0*x(M+1)*cos(theta));
%     for m=M:-1:1
%         A(m)=A(m+1)*(exp(j*kx(m+1)*x(m))+R(m+1)*exp(-j*kx(m+1)*x(m)))/(exp(j*kx(m)*x(m))+R(m)*exp(-j*kx(m)*x(m)));
%         Ez(m,p)=A(m)*(exp(j*kx(m)*x(m))+R(m)*exp(-j*kx(m)*x(m)));
%     end
end

theta=angle*180/pi;
%plot(x,R)
%plot(theta,abs(Rmplus1),'-x')
%ylim([0,1])