clear
% compares actual Ez fields with discrete approximation
c=299792458;
u0=4*pi*1e-7;
e0=1/(c^2*u0);
f=30e9;
lambda=c/f;
L=5*lambda;
k0=2*pi/lambda;
n=240;  M=24000;
n_thetas=10;
[R_actual,theta,Ez]=analytical_Ez(M,n_thetas);
[R1,phis1,theta,avg_time_linear]=linearFEM_Ez(n+1,n_thetas);
[R2,phis2,theta,avg_time_quad]=quadraticFEM_Ez(n+1,n_thetas);
[R3,phis3,theta,avg_time_cubic]=cubicFEM_Ez(n+1,n_thetas);
[R4,phis4,theta,avg_time_quartic]=quarticFEM_Ez(n+1,n_thetas);
x=linspace(0,L,n+1)';
x1=linspace(0,L,M+1)';

figure
Ez(1,:)=0.0+j*0.0;
plot(x1,Ez(:,end/2),x,phis1(:,end/2),x,phis2(:,end/2),x,phis3(:,end/2),x,phis4(:,end/2))
title('Discrete Approximation of E_{z} field compared to FEM solutions')
legend('discrete approximation','linear','quadratic','cubic','quartic')
xlabel('x')
ylabel('E_{z}')
grid on

for i=1:n+1
    err1(i)=abs(phis1(i,end/2)-Ez((i-1)*100+1,end/2));
    err2(i)=abs(phis2(i,end/2)-Ez((i-1)*100+1,end/2));
    err3(i)=abs(phis3(i,end/2)-Ez((i-1)*100+1,end/2));
    err4(i)=abs(phis4(i,end/2)-Ez((i-1)*100+1,end/2));
end

figure
plot(x/lambda,log(err1),x/lambda,log(err2),x/lambda,log(err3),x/lambda,log(err4))
title('Relative Error of E_{z} fields along x-axis')
legend('linear','quadratic','cubic','quartic')
xlabel('x/lambda')
ylabel('Log(|error|)')
grid on
ylim([-20,0])