%FEM Method for 1D
%Domain discretization
%N = number of elements
%N+1 = number of nodes
% in the layered region, there are different equations than in the vacuum.

%define constants
c = 299792458;
u0 = 4*pi*1e-7;
e0 = 1/(c^2*u0);
f = 30e9;
lambda = c/f;
E0 = 1
L = 5*lambda; %thickness of dielectric region

N = 103 %number of nodes
M = (N-1)/3 %number of elements, for linear
x = linspace(0, L, N) 
y = linspace((x(3)+x(2))/2, (x(N-1)+x(N-2))/2,M) %center of each element
le = y(2) - y(1) %pg 50, length of each element
er = 4+(2-j*0.1)*(1-y/L).^2;%pg 66. calculate er in each element, not at each node.
ur = 2-j*0.1;%pg 66
k0 = f*2*pi*sqrt(u0*e0)

%selection of interpolation functions
%element matrix k, element vector f, global matrix kk, global vector ff

angle =linspace(0,90,10) * pi/180
for p = 1:10
    theta = angle(p)

    b = zeros(N,1) %since the forcing function is 0, b is zero until the second boundary condition
    alpha = 1/ur
    beta = -k0^2*(er-1/ur*sin(theta)^2)
	kk = zeros(N,N) % global k matrix
    k = [[0,0];[0,0]] %elemental k matrix

for i = 1:M %this is the entire layered region. calculate k, the element matrice, for every element (layer).
    x1 = x(3*i-2)
    x2 = x(3*i-1)
    x3 = x(3*i)
    x4 = x(3*i+1)
     N1 = @(x) (x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4)) % pg 73
     N2 = @(x) (x-x1).*(x-x3).*(x-x4)/((x2-x1)*(x2-x3)*(x2-x4)) 
     N3 = @(x) (x-x1).*(x-x2).*(x-x4)/((x3-x1)*(x3-x2)*(x3-x4)) 
     N4 = @(x) (x-x1).*(x-x2).*(x-x3)/((x4-x1)*(x4-x2)*(x4-x3))
     dN1 = @(x) (3*x.^2-2*x*x2-2*x*x3-2*x*x4+x2*x3+x2*x4+x3*x4)/((x1-x2)*(x1-x3)*(x1-x4)) % pg 73
     dN2 = @(x) (3*x.^2-2*x*x1-2*x*x3-2*x*x4+x1*x3+x1*x4+x3*x4)/((x2-x1)*(x2-x3)*(x2-x4)) 
     dN3 = @(x) (3*x.^2-2*x*x2-2*x*x1-2*x*x4+x2*x1+x2*x4+x1*x4)/((x3-x1)*(x3-x2)*(x3-x4)) 
     dN4 = @(x) (3*x.^2-2*x*x2-2*x*x3-2*x*x1+x2*x3+x2*x1+x3*x1)/((x4-x1)*(x4-x2)*(x4-x3))
     %     beta is a function of x. instead of approximating x, assign it to be
%     a function of x.
     %beta = @(x) -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2)
    k11 = quadgr(@(x) alpha*(3*x.^2-2*x*x2-2*x*x3-2*x*x4+x2*x3+x2*x4+x3*x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(3*x.^2-2*x*x2-2*x*x3-2*x*x4+x2*x3+x2*x4+x3*x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2).*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4)),x1,x4); % pg 75;
    k12 = quadgr(@(x) alpha*(3*x.^2-2*x*x2-2*x*x3-2*x*x4+x2*x3+x2*x4+x3*x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(3*x.^2-2*x*x1-2*x*x3-2*x*x4+x1*x3+x1*x4+x3*x4)/((x2-x1)*(x2-x3)*(x2-x4))...
        -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2).*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(x-x1).*(x-x3).*(x-x4)/((x2-x1)*(x2-x3)*(x2-x4)) ,x1,x4); % pg 75;
    k13 = quadgr(@(x) alpha*(3*x.^2-2*x*x2-2*x*x3-2*x*x4+x2*x3+x2*x4+x3*x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(3*x.^2-2*x*x2-2*x*x1-2*x*x4+x2*x1+x2*x4+x1*x4)/((x3-x1)*(x3-x2)*(x3-x4)) ...
        -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2).*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(x-x1).*(x-x2).*(x-x4)/((x3-x1)*(x3-x2)*(x3-x4)),x1,x4); % pg 75;
    k14 = quadgr(@(x) alpha*(3*x.^2-2*x*x2-2*x*x3-2*x*x4+x2*x3+x2*x4+x3*x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(3*x.^2-2*x*x2-2*x*x3-2*x*x1+x2*x3+x2*x1+x3*x1)/((x4-x1)*(x4-x2)*(x4-x3))...
        -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2).*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(x-x1).*(x-x2).*(x-x3)/((x4-x1)*(x4-x2)*(x4-x3)),x1,x4); % pg 75;
  
    k21 = k12
    k31 = k13
    k41 = k14
    k34 = k12
    k43 = k34
    k24 = k13
    k42 = k24
    k44 = k11
    k22 = quadgr(@(x) alpha*(3*x.^2-2*x*x1-2*x*x3-2*x*x4+x1*x3+x1*x4+x3*x4)/((x2-x1)*(x2-x3)*(x2-x4))...
        .*(3*x.^2-2*x*x1-2*x*x3-2*x*x4+x1*x3+x1*x4+x3*x4)/((x2-x1)*(x2-x3)*(x2-x4))...
        -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2).*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(x-x1).*(x-x3).*(x-x4)/((x2-x1)*(x2-x3)*(x2-x4)), x1, x4); % pg 75;
    k23 = quadgr(@(x) alpha*(3*x.^2-2*x*x1-2*x*x3-2*x*x4+x1*x3+x1*x4+x3*x4)/((x2-x1)*(x2-x3)*(x2-x4))...
        .*(3*x.^2-2*x*x2-2*x*x1-2*x*x4+x2*x1+x2*x4+x1*x4)/((x3-x1)*(x3-x2)*(x3-x4)) ...
        -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2).*(x-x2).*(x-x3).*(x-x4)/((x1-x2)*(x1-x3)*(x1-x4))...
        .*(x-x1).*(x-x2).*(x-x4)/((x3-x1)*(x3-x2)*(x3-x4)),x1,x4); % pg 75;
    k32 = k23
    k33 = k22
    
    
    %combining the element matrices with the global k matrix
    kk(3*i-2,3*i-2) = kk(3*i-2,3*i-2) + k11;
    kk(3*i-1,3*i-2) = kk(3*i-1,3*i-2) + k21;
    kk(3*i-0,3*i-2) = kk(3*i-0,3*i-2) + k31;
    kk(3*i+1,3*i-2) = kk(3*i+1,3*i-2) + k41;
    kk(3*i-2,3*i-1) = kk(3*i-2,3*i-1) + k12;
    kk(3*i-1,3*i-1) = kk(3*i-1,3*i-1) + k22;
    kk(3*i-0,3*i-1) = kk(3*i-0,3*i-1) + k32;
    kk(3*i+1,3*i-1) = kk(3*i+1,3*i-1) + k42;
    kk(3*i-2,3*i-0) = kk(3*i-2,3*i-0) + k13;
    kk(3*i-1,3*i-0) = kk(3*i-1,3*i-0) + k23;
    kk(3*i-0,3*i-0) = kk(3*i-0,3*i-0) + k33;
    kk(3*i+1,3*i-0) = kk(3*i+1,3*i-0) + k43;    
    kk(3*i-2,3*i+1) = kk(3*i-2,3*i+1) + k14;
    kk(3*i-1,3*i+1) = kk(3*i-1,3*i+1) + k24;
    kk(3*i-0,3*i+1) = kk(3*i-0,3*i+1) + k34;
    kk(3*i+1,3*i+1) = kk(3*i+1,3*i+1) + k44;
end

q = 2*j*k0*E0*cos(theta)*exp(j*k0*L*cos(theta))% pg 65
gamma = j*k0*cos(theta) %pg 65

%enforce boundary conditions
kk(1,1) = 1
kk(1,2:N) = 0 % b(1) = 0 is implied by this neumann BC
kk(N,N) = kk(N,N)+gamma %pg 56
b(N) = q %pg 56
phi = kk\b

R(p) = (phi(length(phi)) - E0*exp(j*k0*L*cos(theta)))./(E0*exp(-j*k0*L*cos(theta)))
end

plot(angle*180/pi,abs(R))
xlabel('Incident Angle \phi [degrees]')
ylabel('Reflection coefficient')
axis([0,90,0,1])
title(['Quadratic FEM solution - N =',num2str(N)])
%semilogy(M,error,'+')
%xlabel('N, number of cells used')
%ylabel('error [dB]')
%title('Problem 8')
 %plot(xL,phiL,x,phi,'r')