%FEM Method for 1D
%Domain discretization
%N = number of elements
%N+1 = number of nodes
% in the layered region, there are different equations than in the vacuum.

%define constants
c = 299792458;
u0 = 4*pi*1e-7;
e0 = 1/(c^2*u0);
f = 30e9;
lambda = c/f;
E0 = 1 
L = 5*lambda; %thickness of dielectric region
N = 100 %number of nodes
M = N-1 %number of elements, for linear
x = linspace(0, L, N) 
y = linspace((x(2)+x(1))/2, (x(N) + x(M))/2,M) %center of each element
le = x(2) - x(1) %pg 50, element length
er = -(4+(2-j*0.1)*(1-y/L).^2);%pg 66. calculate er in each element, not at each node.
ur = -(2-j*0.1);%pg 66
k0 = f*2*pi*sqrt(u0*e0)

%selection of interpolation functions
%element matrix k, element vector f, global matrix kk, global vector ff

angle =linspace(0,90,10) * pi/180
for p = 1:10
    theta = angle(p)

    b = zeros(N,1) %since the forcing function is 0, b is zero until the second boundary condition
    alpha = 1/ur
    beta = -k0^2*(er-1/ur*sin(theta)^2)
	kk = zeros(N,N) % global k matrix
    k = [[0,0];[0,0]] %elemental k matrix

for i = 1:M %this is the entire layered region. calculate k, the element matrice, for every element (layer).
    x1 = x(i)
    x2 = x(i+1)
    N1 = @(x) (x2 - x)/le % pg 50
    N2 = @(x) (x - x1)/le % pg 50
    dN1 = -1/le %take derivative of N1
    dN2 = 1/le %take derivative of N2;
    k11 = quadgr(@(x) alpha*dN1*dN1+beta(i)*(x2 - x)/le.*(x2-x)/le,x1,x2); % pg 51;
    k12 = quadgr(@(x) alpha*dN1*dN2+beta(i)*(x2 - x)/le.*(x-x1)/le,x1,x2); % pg 51
    k22 = k11;
    k21 = k12;
    %combining the element matrices with the global k matrix
    kk(i,i) = kk(i,i) + k11;
    kk(i,i+1) = kk(i,i+1) + k12;
    kk(i+1,i) = kk(i+1,i) + k21;
    kk(i+1,i+1) = kk(i+1,i+1) + k22;
end

q = 2*j*k0*E0*cos(theta)*exp(j*k0*L*cos(theta))% pg 65
gamma = j*k0*cos(theta) %pg 65

%enforce boundary conditions
kk(1,1) = 1
kk(1,2:N) = 0 % b(1) = 0 is implied by this dirichlet BC
kk(N,N) = alpha/le+beta(M)*le/(3)+gamma %pg 56
%kk(N,1:N-1) = 0
b(N) = q %pg 56
phi = kk\b

R(p) = (phi(length(phi)) - E0*exp(j*k0*L*cos(theta)))./(E0*exp(-j*k0*L*cos(theta)))
end

plot(angle*180/pi,abs(R))
xlabel('Incident Angle \phi [degrees]')
ylabel('Reflection coefficient')
axis([0,90,0,1])
title(['Linear FEM solution - N =',num2str(N)])
%semilogy(M,error,'+')
%xlabel('N, number of cells used')
%ylabel('error [dB]')
%title('Problem 8')