function [R,phis,theta,avg_time_linear]=linearFEM_Ez(n,n_thetas)
% 1D FEM program which uses linear interpolation functions
% for the general 1-D equation:
% -d/dx(a*dphi/dx)+b*phi=f, 0<x<L
% phi is Ez vector
% n - number of nodes
% m - number of elements    

% set constants
c=299792458;
u0=4*pi*1e-7;
e0=1/(c^2*u0);
f=30e9;
lambda=c/f;
L=5*lambda;
E0=1;
k0=2*pi/lambda;
% set nodes
x=linspace(0,L,n)';
m=(n-1);
% get 1D quadrature rules
[pts,wghts]=gaussian_quad(4);
% set functions for alpha and beta using eps_r and mu_r
er=@(x) 4+(2-j*0.1)*(1-x/L).^2;     ur=2-j*0.1;
a=1/ur;     b=@(x,theta) -(k0^2)*(er(x)-(sin(theta)^2)/ur); 
% set functions for N_i and dN_i/dx for i=1,2
N1 = @(x,x1,x2) (x-x2)./((x1-x2)); 
N2 = @(x,x1,x2) (x-x1)./((x2-x1)); 
dN1 = @(x,x1,x2) 1/(x1-x2);
dN2 = @(x,x1,x2) 1/(x2-x1);
angle=linspace(0,90,n_thetas)*pi/180';
for thetas=1:n_thetas
    tic
    theta1=angle(thetas);
    K=zeros(n,n);
    B=zeros(n,1);
    for count=1:m
        x1=x(count);    x2=x(count+1); 
        % recompute quadrature nodes and weights for the interval [x1,x3]
        pts1=pts*(x2-x1)+x1;
        wghts1=wghts*(x2-x1);
        % element matrix entries are computed numerically using
        % Gaussian quadrature
        ke(1,1)=sum(wghts1.*(a*dN1(pts1,x1,x2).*dN1(pts1,x1,x2)+b(pts1,theta1).*N1(pts1,x1,x2).*N1(pts1,x1,x2)));
        ke(1,2)=sum(wghts1.*(a*dN1(pts1,x1,x2).*dN2(pts1,x1,x2)+b(pts1,theta1).*N1(pts1,x1,x2).*N2(pts1,x1,x2)));
        ke(2,1)=ke(1,2); ke(2,2)=ke(1,1);
        K(count:count+1,count:count+1)=K(count:count+1,count:count+1)+ke;
    end
    q=2*j*k0*E0*cos(theta1)*exp(j*k0*L*cos(theta1));
    gamma=j*k0*cos(theta1);
    % Impose boundary conditions
    K(1,:)=0;   K(:,1)=0;   K(1,1)=1;   B(1)=0;
    K(n,n)=K(n,n)+gamma;
    B(end)=q;
    % Solve linear system
    phi=K\B;
    phis(:,thetas)=phi;
    R(thetas)=(phi(end)-E0*exp(j*k0*L*cos(theta1)))./(E0*exp(-j*k0*L*cos(theta1)));
    times(thetas)=toc;
end
avg_time_linear=mean(times)
theta=angle*180/pi;
%figure
%plot(theta,abs(R))
%xlabel('Incident Angle \phi [degrees]')
%ylabel('Reflection coefficient')
%axis([0,90,0,1])
