%FEM Method for 1D
%Domain discretization
%N = number of elements
%N+1 = number of nodes

c = 299792458;
u0 = 4*pi*1e-7;
e0 = 1/(c^2*u0);
f = 30e9;
lambda = c/f;
E0 = 1 
L = 5*lambda; %thickness of dielectric region
N = 50 %number of nodes
M = N-1 %number of elements, for linear
x = linspace(0, L, N) 
le = x(2) - x(1) %pg 50, element length
er = 4+(2-j*0.1)*(1-x/L).^2;%pg 66. calculate er in each element, not at each node.
er = er(1:M)
ur = 2-j*0.1;%pg 66
k0 = f*2*pi*sqrt(u0*e0)

theta = pi/3
 b = zeros(N,1) %since the forcing function is 0, b is zero until the second boundary condition
    alpha = 1/ur
    beta = -k0^2*(er-1/ur*sin(theta)^2)
	kk = zeros(N,N) % global k matrix
    k = [[0,0];[0,0]] %elemental k matrix

for i = 1:N-1
    x1 = x(i)
    x2 = x(i+1)
    N1 = @(x) (x2 - x)/le
    N2 = @(x) (x - x1)/le
    dN1 = -1/le
    dN2 = 1/le
    k11 = quadgr(@(x) alpha*dN1*dN1+beta(i)/le^2*(x2 - x).*(x2-x),x1,x2)
    k12 = quadgr(@(x) alpha*dN1*dN2+beta(i)/le^2*(x2 - x).*(x-x1),x1,x2)
    k22 = k11
    k21 = k12
    kk(i,i) = kk(i,i) + k11
    kk(i,i+1) = kk(i,i+1) + k12
    kk(i+1,i) = kk(i+1,i) + k21
    kk(i+1,i+1) = kk(i+1,i+1) + k22
end

q = 2*k0*cos(theta)*exp(j*k0*cos(theta))*E0% pg 65
gamma = j*k0*cos(theta) %pg 65

%enforce boundary conditions
kk(1,1) = 1
kk(1,2:N) = 0 % b(1) = 0 is implied by this dirichlet BC
kk(N,N) = alpha/le+beta(M)*le/(3)+gamma %pg 56
kk(N,1:N-1) = 0
b(N) = q %pg 56
phi = kk\b
Rfem = (phi(length(phi)) - E0*exp(j*k0*L*cos(theta)))./(E0*exp(-j*k0*L*cos(theta)))
Rfem = linspace(Rfem,Rfem,N)
% y = linspace(0,1,100)
% exact = sin(sqrt(2)*y)/sin(sqrt(2))
% plot(x,phi,'h',y,exact)
% xlabel('X')
% ylabel('phi')
% title(['Problem 8 - FEM method with N = ', num2str(N), ' nodes'])
% axis([0,1,0,1])
% legend('FEM method', 'Analytical')

er = 4+(2-j*0.1)*(1-x/L).^2;%pg 66. calculate er in each element, not at each node.

k = f*2*pi*sqrt(u0*e0)*sqrt(ur*er-sin(theta)^2);
R = zeros(1,N);
n = zeros(1,N);
R(1) = -1;

for i =1:N-1
    n(i+1) = (ur*k(i+1)-ur*k(i))./(ur*k(i+1)+ur*k(i));
    R(i+1) = exp(2*j*k(i+1)*x(i+1)).*(n(i+1)+R(i).*exp(-2*j*k(i).*x(i+1)))/(1+n(i+1)*R(i).*exp(-2*j*k(i).*x(i+1)));
    
end
plot(x,R,x,Rfem,'h')
%angle = linspace(0,90,91)
%real(R(length(R)))
%plot(angle,Rmplus1)
