%FEM Method for 1D
%Domain discretization
%N = number of elements
%N+1 = number of nodes
% in the layered region, there are different equations than in the vacuum.

%define constants
c = 299792458;
u0 = 4*pi*1e-7;
e0 = 1/(c^2*u0);
f = 30e9;
lambda = c/f;
H0 = 1
L = 5*lambda; %thickness of dielectric region

N = 101 %number of nodes
M = (N-1)/2 %number of elements, for linear
x = linspace(0, L, N) 
y = linspace(x(2), x(N-1),M) %center of each element
le = y(2) - y(1) %pg 50, length of each element
er = 4+(2-j*0.1)*(1-y/L).^2;%pg 66. calculate er in each element, not at each node.
ur = 2-j*0.1;%pg 66
k0 = f*2*pi*sqrt(u0*e0)

%selection of interpolation functions
%element matrix k, element vector f, global matrix kk, global vector ff

angle =linspace(0,90,10) * pi/180
for p = 1:10
    theta = angle(p)

    b = zeros(N,1) %since the forcing function is 0, b is zero until the second boundary condition
    alpha = 1./er
    beta = -k0^2.*(ur-1./er*sin(theta)^2)
	kk = zeros(N,N) % global k matrix
    k = [[0,0];[0,0]] %elemental k matrix

for i = 1:M %this is the entire layered region. calculate k, the element matrice, for every element (layer).
    x1 = x(2*i-1)
    x2 = x(2*i)
    x3 = x(2*i+1)
     N1 = @(x) (x-x2).*(x-x3)/(x1-x2)/(x1-x3) % pg 73
     N2 = @(x) (x-x1).*(x-x3)/(x2-x1)/(x2-x3)% pg 73
     N3 = @(x) (x-x1).*(x-x2)/(x3-x1)/(x3-x2)
     dN1 = @(x) (2*x - x2-x3)/(x1-x2)/(x1-x3) %take derivative of N1
     dN2 = @(x) (2*x - x1-x3)/(x2-x1)/(x2-x3) %take derivative of N2;
     dN3 = @(x) (2*x - x1-x2)/(x3-x1)/(x3-x2) %take derivative of N2;
%     beta is a function of x. instead of approximating x, assign it to be
%     a function of x.
   %  beta = @(x) -k0^2*(4+(2-j*0.1)*(1-x/L).^2-1/ur*sin(theta)^2)
    k11 = quadgr(@(x) 1./(4+(2-j*0.1).*(1-x/L).^2).*(2*x - x2-x3)/(x1-x2)/(x1-x3).*(2*x - x2-x3)/(x1-x2)/(x1-x3)-k0^2.*(ur-1./(4+(2-j*0.1)*(1-x/L).^2)*sin(theta)^2).*(x-x2).*(x-x3)/(x1-x2)/(x1-x3).*(x-x2).*(x-x3)/(x1-x2)/(x1-x3),x1,x3); % pg 75;
    %k11 = quadgr(@(x) alpha*dN1*dN1 + beta(x)*N1*N1,x1,x3)
    k12 = quadgr(@(x) 1./(4+(2-j*0.1).*(1-x/L).^2).*(2*x - x2-x3)/(x1-x2)/(x1-x3).*(2*x - x1-x3)/(x2-x1)/(x2-x3)-k0^2.*(ur-1./(4+(2-j*0.1)*(1-x/L).^2)*sin(theta)^2).*(x-x2).*(x-x3)/(x1-x2)/(x1-x3).*(x-x1).*(x-x3)/(x2-x1)/(x2-x3),x1,x3); % pg 75;
    k13 = quadgr(@(x) 1./(4+(2-j*0.1).*(1-x/L).^2).*(2*x - x2-x3)/(x1-x2)/(x1-x3).*(2*x - x1-x2)/(x3-x1)/(x3-x2)-k0^2.*(ur-1./(4+(2-j*0.1)*(1-x/L).^2)*sin(theta)^2).*(x-x2).*(x-x3)/(x1-x2)/(x1-x3).*(x-x1).*(x-x2)/(x3-x1)/(x3-x2),x1,x3); % pg 75;
    k21 = k12 
    k22 = quadgr(@(x) 1./(4+(2-j*0.1).*(1-x/L).^2).*(2*x - x1-x3)/(x2-x1)/(x2-x3).*(2*x - x1-x3)/(x2-x1)/(x2-x3)-k0^2.*(ur-1./(4+(2-j*0.1)*(1-x/L).^2)*sin(theta)^2).*(x-x1).*(x-x3)/(x2-x1)/(x2-x3).*(x-x1).*(x-x3)/(x2-x1)/(x2-x3),x1,x3); % pg 75;
    k23 = quadgr(@(x) 1./(4+(2-j*0.1).*(1-x/L).^2).*(2*x - x1-x3)/(x2-x1)/(x2-x3).*(2*x - x1-x2)/(x3-x1)/(x3-x2)-k0^2.*(ur-1./(4+(2-j*0.1)*(1-x/L).^2)*sin(theta)^2).*(x-x1).*(x-x3)/(x2-x1)/(x2-x3).*(x-x1).*(x-x2)/(x3-x1)/(x3-x2),x1,x3); % pg 75;
    k31 = k13
    k32 = k23
    k33 = quadgr(@(x) 1./(4+(2-j*0.1).*(1-x/L).^2).*(2*x - x1-x2)/(x3-x1)/(x3-x2).*(2*x - x1-x2)/(x3-x1)/(x3-x2)-k0^2.*(ur-1./(4+(2-j*0.1)*(1-x/L).^2)*sin(theta)^2).*(x-x1).*(x-x2)/(x3-x1)/(x3-x2).*(x-x1).*(x-x2)/(x3-x1)/(x3-x2),x1,x3); % pg 75;
%  
%used matrix from pg 76, since alpha and beta can be appxorimated within
%the element
%     k11 = alpha*7/(3*le) + beta(i)*2*le/15
%     k12 = -alpha*8/(3*le)+beta(i)*le/15
%     k13 = alpha/(3*le)-beta(i)*le/30
%     k21 = k12
%     k23 = k12
%     k32 = k12
%     k33 = k11
%     k31 = k13
%     k22 = alpha*16/(3*le)+beta(i)*8*le/15
    %combining the element matrices with the global k matrix
    kk(2*i-1,2*i-1) = kk(2*i-1,2*i-1) + k11;
    kk(2*i-1,2*i)   = kk(2*i-1,2*i) + k12;
    kk(2*i-1,2*i+1) = kk(2*i-1,2*i+1) + k13;
    kk(2*i+0,2*i-1) = kk(2*i+0,2*i-1) + k21;
    kk(2*i+0,2*i)   = kk(2*i+0,2*i) + k22;
    kk(2*i+0,2*i+1) = kk(2*i+0,2*i+1) + k23;
    kk(2*i+1,2*i-1) = kk(2*i+1,2*i-1) + k31;
    kk(2*i+1,2*i)   = kk(2*i+1,2*i) + k32;
    kk(2*i+1,2*i+1) = kk(2*i+1,2*i+1) + k33;
end

q = 2*j*k0*H0*cos(theta)*exp(j*k0*L*cos(theta))% pg 65
gamma = j*k0*cos(theta) %pg 65

%enforce boundary conditions
kk(1,1) = 1
kk(1,2:N) = 0 % b(1) = 0 is implied by this neumann BC
kk(N,N) = kk(N,N)+gamma %pg 56
b(N) = q %pg 56
phi = kk\b

R(p) = (phi(length(phi)) - H0*exp(j*k0*L*cos(theta)))./(H0*exp(-j*k0*L*cos(theta)))
end

plot(angle*180/pi,abs(R))
xlabel('Incident Angle \phi [degrees]')
ylabel('Reflection coefficient')
axis([0,90,0,1])
title(['Quadratic FEM solution - N =',num2str(N)])
%semilogy(M,error,'+')
%xlabel('N, number of cells used')
%ylabel('error [dB]')
%title('Problem 8')
 %plot(xL,phiL,x,phi,'r')