function [R,phis,theta,avg_time_quartic]=quarticFEM_Ez(n,n_thetas)
% 1D FEM program which uses quartic interpolation functions
% for the general 1-D equation:
% -d/dx(a*dphi/dx)+b*phi=f, 0<x<L
% phi is Ez vector
% n - number of nodes
% m - number of elements    

% set constants
c=299792458;
u0=4*pi*1e-7;
e0=1/(c^2*u0);
f=30e9;
lambda=c/f;
L=5*lambda;
E0=1;
k0=2*pi/lambda;
% set nodes
x=linspace(0,L,n)';
m=(n-1)/4;
% get 1D quadrature rules
[pts,wghts]=gaussian_quad(10);
% set functions for alpha and beta using eps_r and mu_r
er=@(x) 4+(2-j*0.1)*(1-x/L).^2;     ur=2-j*0.1;
a=1/ur;     b=@(x,theta) -(k0^2)*(er(x)-(sin(theta)^2)/ur); 
% set functions for N_i and dN_i/dx for i=1,2,3,4,5
N1 = @(x,x1,x2,x3,x4,x5) (x-x2).*(x-x3).*(x-x4).*(x-x5)/((x1-x2)*(x1-x3)*(x1-x4)*(x1-x5)); 
N2 = @(x,x1,x2,x3,x4,x5) (x-x1).*(x-x3).*(x-x4).*(x-x5)/((x2-x1)*(x2-x3)*(x2-x4)*(x2-x5)); 
N3 = @(x,x1,x2,x3,x4,x5) (x-x1).*(x-x2).*(x-x4).*(x-x5)/((x3-x1)*(x3-x2)*(x3-x4)*(x3-x5)); 
N4 = @(x,x1,x2,x3,x4,x5) (x-x1).*(x-x2).*(x-x3).*(x-x5)/((x4-x1)*(x4-x2)*(x4-x3)*(x4-x5));
N5 = @(x,x1,x2,x3,x4,x5) (x-x1).*(x-x2).*(x-x3).*(x-x4)/((x5-x1)*(x5-x2)*(x5-x3)*(x5-x4));
dN1 = @(x,x1,x2,x3,x4,x5) ((x-x2).*(x-x3).*(x-x4)+(x-x2).*(x-x3).*(x-x5)+(x-x2).*(x-x4).*(x-x5)+(x-x3).*(x-x4).*(x-x5))/((x1-x2)*(x1-x3)*(x1-x4)*(x1-x5)); 
dN2 = @(x,x1,x2,x3,x4,x5) ((x-x1).*(x-x3).*(x-x4)+(x-x1).*(x-x3).*(x-x5)+(x-x1).*(x-x4).*(x-x5)+(x-x3).*(x-x4).*(x-x5))/((x2-x1)*(x2-x3)*(x2-x4)*(x2-x5)); 
dN3 = @(x,x1,x2,x3,x4,x5) ((x-x1).*(x-x2).*(x-x4)+(x-x1).*(x-x2).*(x-x5)+(x-x1).*(x-x4).*(x-x5)+(x-x2).*(x-x4).*(x-x5))/((x3-x1)*(x3-x2)*(x3-x4)*(x3-x5)); 
dN4 = @(x,x1,x2,x3,x4,x5) ((x-x2).*(x-x3).*(x-x1)+(x-x2).*(x-x3).*(x-x5)+(x-x2).*(x-x1).*(x-x5)+(x-x3).*(x-x1).*(x-x5))/((x4-x1)*(x4-x2)*(x4-x3)*(x4-x5)); 
dN5 = @(x,x1,x2,x3,x4,x5) ((x-x2).*(x-x3).*(x-x4)+(x-x2).*(x-x3).*(x-x1)+(x-x2).*(x-x4).*(x-x1)+(x-x3).*(x-x4).*(x-x1))/((x5-x1)*(x5-x2)*(x5-x3)*(x5-x4)); 
angle=linspace(0,90,n_thetas)*pi/180';
for thetas=1:n_thetas
    tic
    theta1=angle(thetas);
    K=zeros(n,n);
    B=zeros(n,1);
    for count=1:m
        x1=x(4*count-3);    x2=x(4*count-2);    x3=x(4*count-1); x4 = x(4*count); x5 = x(4*count+1);
        % recompute quadrature nodes and weights for the interval [x1,x3]
        pts1=pts*(x5-x1)+x1;
        wghts1=wghts*(x5-x1);
        % element matrix entries are computed numerically using
        % Gaussian quadrature
        ke(1,1)=sum(wghts1.*(a*dN1(pts1,x1,x2,x3,x4,x5).*dN1(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N1(pts1,x1,x2,x3,x4,x5).*N1(pts1,x1,x2,x3,x4,x5)));
        ke(1,2)=sum(wghts1.*(a*dN1(pts1,x1,x2,x3,x4,x5).*dN2(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N1(pts1,x1,x2,x3,x4,x5).*N2(pts1,x1,x2,x3,x4,x5)));
        ke(1,3)=sum(wghts1.*(a*dN1(pts1,x1,x2,x3,x4,x5).*dN3(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N1(pts1,x1,x2,x3,x4,x5).*N3(pts1,x1,x2,x3,x4,x5)));
        ke(1,4)=sum(wghts1.*(a*dN1(pts1,x1,x2,x3,x4,x5).*dN4(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N1(pts1,x1,x2,x3,x4,x5).*N4(pts1,x1,x2,x3,x4,x5)));
        ke(1,5)=sum(wghts1.*(a*dN1(pts1,x1,x2,x3,x4,x5).*dN5(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N1(pts1,x1,x2,x3,x4,x5).*N5(pts1,x1,x2,x3,x4,x5)));
        ke(5,1)=ke(1,5); ke(4,1)=ke(1,4); ke(3,1)=ke(1,3); ke(2,1)=ke(1,2);
        ke(5,5)=sum(wghts1.*(a*dN5(pts1,x1,x2,x3,x4,x5).*dN5(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N5(pts1,x1,x2,x3,x4,x5).*N5(pts1,x1,x2,x3,x4,x5)));
        ke(2,2)=sum(wghts1.*(a*dN2(pts1,x1,x2,x3,x4,x5).*dN2(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N2(pts1,x1,x2,x3,x4,x5).*N2(pts1,x1,x2,x3,x4,x5)));
        ke(2,3)=sum(wghts1.*(a*dN2(pts1,x1,x2,x3,x4,x5).*dN3(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N2(pts1,x1,x2,x3,x4,x5).*N3(pts1,x1,x2,x3,x4,x5)));
        ke(2,4)=sum(wghts1.*(a*dN2(pts1,x1,x2,x3,x4,x5).*dN4(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N2(pts1,x1,x2,x3,x4,x5).*N4(pts1,x1,x2,x3,x4,x5)));
        ke(2,5)=sum(wghts1.*(a*dN2(pts1,x1,x2,x3,x4,x5).*dN5(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N2(pts1,x1,x2,x3,x4,x5).*N5(pts1,x1,x2,x3,x4,x5)));
        ke(5,2)=ke(2,5); ke(4,2)=ke(2,4); ke(3,2)=ke(2,3);
        ke(3,3)=sum(wghts1.*(a*dN3(pts1,x1,x2,x3,x4,x5).*dN3(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N3(pts1,x1,x2,x3,x4,x5).*N3(pts1,x1,x2,x3,x4,x5)));
        ke(3,4)=sum(wghts1.*(a*dN3(pts1,x1,x2,x3,x4,x5).*dN4(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N3(pts1,x1,x2,x3,x4,x5).*N4(pts1,x1,x2,x3,x4,x5)));
        ke(3,5)=sum(wghts1.*(a*dN3(pts1,x1,x2,x3,x4,x5).*dN5(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N3(pts1,x1,x2,x3,x4,x5).*N5(pts1,x1,x2,x3,x4,x5)));
        ke(5,3)=ke(3,5); ke(4,3)=ke(3,4);
        ke(4,4)=sum(wghts1.*(a*dN4(pts1,x1,x2,x3,x4,x5).*dN4(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N4(pts1,x1,x2,x3,x4,x5).*N4(pts1,x1,x2,x3,x4,x5)));
        ke(4,5)=sum(wghts1.*(a*dN4(pts1,x1,x2,x3,x4,x5).*dN5(pts1,x1,x2,x3,x4,x5)+b(pts1,theta1).*N4(pts1,x1,x2,x3,x4,x5).*N5(pts1,x1,x2,x3,x4,x5)));
        ke(5,4)=ke(4,5);
        % fill global K matrix and b vector
        K(4*count-3:4*count+1,4*count-3:4*count+1)=K(4*count-3:4*count+1,4*count-3:4*count+1)+ke;
    end
    q=2*j*k0*E0*cos(theta1)*exp(j*k0*L*cos(theta1));
    gamma=j*k0*cos(theta1);
    % Impose boundary conditions
    K(1,:)=0;   K(:,1)=0;   K(1,1)=1;   B(1)=0;
    K(n,n)=K(n,n)+gamma;
    B(end)=q;
    % Solve linear system
    phi=K\B;
    phis(:,thetas)=phi;
    %tic
    %phi1=qmr(K,B,1e-6,2*length(K));
    %times2(thetas)=toc;
    
    %tic
    %phi2=bicg(K,B,1e-6,2*length(K));
    %times3(thetas)=toc;
    
    R(thetas)=(phi(end)-E0*exp(j*k0*L*cos(theta1)))./(E0*exp(-j*k0*L*cos(theta1)));
    times(thetas)=toc;
end
avg_time_quartic=mean(times)
%avg_time_quartic2=mean(times2);
%avg_time_quartic3=mean(times3);
theta=angle*180/pi;
%figure
%plot(theta,abs(R))
%xlabel('Incident Angle \phi [degrees]')
%ylabel('Reflection coefficient')
%axis([0,90,0,1])
